# -*- coding: utf-8 -*-

import requests
from .network_client import NetworkClient
from .style_client import StyleClient
from .algorithm_client import LayoutClient
from .algorithm_client import EdgeBundlingClient
from .session_client import SessionClient

from . import PORT, IP, VERSION


class CyRestClient(object):

    def __init__(self, ip=IP, port=PORT, version=VERSION):
        self.__url = 'http://' + ip + ':' + str(port) + '/' + version + '/'
        self.network = NetworkClient(self.__url)
        self.style = StyleClient(self.__url)
        self.layout = LayoutClient(self.__url)
        self.edgebundling = EdgeBundlingClient(self.__url)
        self.session = SessionClient(self.__url)

    def status(self):
        try:
            response = requests.get(self.__url).json()
        except Exception as e:
            print('Could not get status from cyREST: ' + e)
        else:
            return response



__author__ = 'kono'


class NetworkUtil(object):

    @staticmethod
    def name2suid(network, obj_type='node'):
        if obj_type is 'node':
            table = network.get_node_table()
        elif obj_type is 'edge':
            table = network.get_edge_table()
        else:
            raise ValueError('No such object type: ' + obj_type)

        subset= table['name']
        name2suid = {}
        for suid in subset.index:
            name2suid[subset[suid]] = suid

        return name2suid




from . import BASE_URL, HEADERS
import requests
import json
import pandas as pd


class Style(object):

    def __init__(self, name):
        # Validate required argument
        if name is None:
            raise ValueError("Style name is required.")
        else:
            self.__name = name

        self.__url = BASE_URL + 'styles/' + str(name) + '/'

    def get_name(self):
        """
        Get immutable name of this Visual Style.

        :return: Style name as string
        """
        return self.__name

    def __get_new_mapping(self, mapping_type, column=None, col_type='String',
                          vp=None):
        if column is None or vp is None:
            raise ValueError('both column name and visual property are required.')

        new_maping = {
            'mappingType': mapping_type,
            'mappingColumn': column,
            'mappingColumnType': col_type,
            'visualProperty': vp
        }

        return new_maping

    def create_discrete_mapping(self, column=None, col_type='String',
                                vp=None, mappings=None):
        self.__call_create_mapping(
            self.__get_discrete(column=column, col_type=col_type, vp=vp,
                                mappings=mappings))

    def create_continuous_mapping(self, column=None, col_type='String',
                                  vp=None, points=None):
        self.__call_create_mapping(
            self.__get_continuous(column=column, col_type=col_type, vp=vp,
                                  points=points))

    def create_passthrough_mapping(self, column=None, col_type='String',
                                   vp=None):
        self.__call_create_mapping(
            self.__get_passthrough(column=column, col_type=col_type, vp=vp))

    def __call_create_mapping(self, mapping):
        url = self.__url + 'mappings'
        requests.post(url, data=json.dumps([mapping]), headers=HEADERS)

    def __get_passthrough(self, column=None, col_type='String', vp=None):
        return self.__get_new_mapping('passthrough', column=column,
                                      col_type=col_type, vp=vp)

    def __get_discrete(self, column=None, col_type='String', vp=None,
                       mappings=None):
        new_mapping = self.__get_new_mapping('discrete', column=column,
                                             col_type=col_type, vp=vp)
        if mappings is None:
            raise ValueError('key-value pair object (mappings) is required.')
        body = [{'key': key, 'value': mappings[key]} for key in mappings.keys()]
        new_mapping['map'] = body
        return new_mapping

    def __get_continuous(self, column=None, col_type='String', vp=None,
                         points=None):
        if points is None:
            raise ValueError('key-value pair object (mappings) is required.')
        new_mapping = self.__get_new_mapping('continuous', column=column,
                                             col_type=col_type, vp=vp)
        new_mapping['points'] = points
        return new_mapping

    def get_mapping(self, vp=None):
        if vp is None:
            raise ValueError('Visual Property ID is required.')

        url = self.__url + 'mappings/' + vp
        return requests.get(url).json()

    def get_mappings(self):
        url = self.__url + 'mappings'
        return requests.get(url).json()

    def get_default(self, vp=None):
        if vp is None:
            raise ValueError('Visual Property ID is required.')

        url = self.__url + 'defaults/' + vp
        key_value_pair = requests.get(url).content
        print(key_value_pair)
        key2 = requests.get(url).json()
        key_value_pair = key2
        return pd.Series({key_value_pair['visualProperty']: key_value_pair[
            'value']})

    def get_defaults(self):
        url = self.__url + 'defaults'
        result = requests.get(url).json()['defaults']
        vals = {entry['visualProperty']: entry['value'] for entry in result}
        return pd.Series(vals)

    def update_defaults(self, key_value_pair):
        body = []
        for key in key_value_pair:
            entry = {
                'visualProperty': key,
                'value': key_value_pair[key]
            }
            body.append(entry)

        url = self.__url + 'defaults'
        requests.put(url, data=json.dumps(body), headers=HEADERS)

    # Delete Methods

    def delete_mapping(self, vp=None):
        if vp is None:
            return

        url = self.__url + 'mappings/' + vp
        requests.delete(url)

    def delete_mappings(self):
        url = self.__url + 'mappings'
        requests.delete(url)


class StyleUtil(object):

    @staticmethod
    def create_2_color_gradient(min=0, max=10, colors=('red', 'green')):

        points = [
            {
                'value': str(min),
                'lesser': colors[0],
                'equal':  colors[0],
                'greater': colors[0],
            },
            {
                'value': str(max),
                'lesser': colors[1],
                'equal': colors[1],
                'greater': colors[1]
            }
        ]

        return points

    @staticmethod
    def create_slope(min=0, max=10, values=(1, 10)):

        points = [
            {
                'value': str(min),
                'lesser': values[0],
                'equal':  values[0],
                'greater': values[0],
            },
            {
                'value': str(max),
                'lesser': values[1],
                'equal': values[1],
                'greater': values[1]
            }
        ]

        return points




import requests

class SessionClient(object):

    def __init__(self, url):
        self.__url = url + 'session'

    def delete(self):
        requests.delete(self.__url)

    def save(self, file_name=None):
        pass

    def load(self, file_name=None):
        pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""

Setup script for py2cytoscape

To install, run:

python setup.py install

"""

from setuptools import setup, find_packages

setup(
    name='py2cytoscape',
    version='0.5.0',
    description='Utilities to use Cytoscape and Cytoscape.js from Python',
    long_description='Collection of tools for using Cytoscape and '
                     'cytoscape.js from Python.  From v0.4.0, '
                     'it includes wrapper for cyREST REST API.',
    author='Keiichiro Ono',
    author_email='kono@ucsd.edu',
    url='https://github.com/idekerlab/py2cytoscape',
    license='MIT License',
    install_requires=[
        'pandas',
        'networkx',
        'requests',
        'python-igraph',
        'pyparsing',
        'scipy'
    ],
    keywords=['data visualization', 'visualization', 'cytoscape',
              'bioinformatics', 'graph', 'network'],
    classifiers=[
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'License :: OSI Approved :: MIT License',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Scientific/Engineering :: Visualization',
    ],
    test_suite='tests',
    packages=find_packages(),
    include_package_data=True,
)
